@@ -19,6 +19,10 @@ Revision History
 
 --*/
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #include "efidebug.h"
 #include "efipart.h"
 #if defined(_M_X64) || defined(__x86_64__) || defined(__amd64__)
@@ -31,13 +35,17 @@ Revision History
 #include "aarch64/efilibplat.h"
 #elif defined (_M_ARM) || defined(__arm__)
 #include "arm/efilibplat.h"
-#elif defined (_M_MIPS64) || defined(__mips64__)
+#elif defined (_M_MIPS64) || defined(__mips64__) || defined(__mips64)
 #include "mips64el/efilibplat.h"
 #elif defined (__riscv) && __riscv_xlen == 64
 #include "riscv64/efilibplat.h"
+#elif defined (__loongarch64)
+#include "loongarch64/efilibplat.h"
 #endif
+#include "legacy/efilib.h"
 #include "efilink.h"
 #include "efirtlib.h"
+#include "efisetjmp.h"
 #include "efistdarg.h"
 #include "pci22.h"
 #include "libsmbios.h"
@@ -53,6 +61,10 @@ extern EFI_BOOT_SERVICES        *BS;
 extern EFI_RUNTIME_SERVICES     *RT;
 #define gRT                      RT
 
+extern EFI_GUID gZeroGuid;
+#define EFI_ZERO_GUID {0x0, 0x0, 0x0, {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0}}
+#define NullGuid gZeroGuid
+
 extern EFI_GUID gEfiDevicePathProtocolGuid;
 #define DevicePathProtocol gEfiDevicePathProtocolGuid
 extern EFI_GUID gEfiDevicePathToTextProtocolGuid;
@@ -83,10 +95,16 @@ extern EFI_GUID gEfiDiskIoProtocolGuid;
 #define DiskIoProtocol gEfiDiskIoProtocolGuid
 extern EFI_GUID gEfiDiskIo2ProtocolGuid;
 #define DiskIo2Protocol gEfiDiskIo2ProtocolGuid
+extern EFI_GUID gEfiDxeServicesTableGuid;
+#define DxeServicesTable gEfiDxeServicesTableGuid
 extern EFI_GUID gEfiSimpleFileSystemProtocolGuid;
 #define FileSystemProtocol gEfiSimpleFileSystemProtocolGuid
+extern EFI_GUID gEfiLoadedImageDevicePathProtocolGuid;
+#define LoadedImageDevicePathProtocol gEfiLoadedImageDevicePathProtocolGuid
 extern EFI_GUID gEfiLoadFileProtocolGuid;
 #define LoadFileProtocol gEfiLoadFileProtocolGuid
+extern EFI_GUID gEfiLoadFile2ProtocolGuid;
+#define LoadFile2Protocol gEfiLoadFile2ProtocolGuid
 extern EFI_GUID gEfiDeviceIoProtocolGuid;
 #define DeviceIoProtocol gEfiDeviceIoProtocolGuid
 extern EFI_GUID VariableStoreProtocol;
@@ -129,6 +147,9 @@ extern EFI_GUID gEfiDriverFamilyOverrideProtocolGuid;
 #define DriverFamilyOverrideProtocol gEfiDriverFamilyOverrideProtocolGuid
 extern EFI_GUID gEfiEbcProtocolGuid;
 
+extern EFI_GUID gEfiMpServicesProtocolGuid;
+#define MpServicesProtocol gEfiMpServicesProtocolGuid
+
 extern EFI_GUID gEfiGlobalVariableGuid;
 #define EfiGlobalVariable gEfiGlobalVariableGuid
 extern EFI_GUID gEfiFileInfoGuid;
@@ -144,7 +165,6 @@ extern EFI_GUID gEfiVT100Guid;
 extern EFI_GUID gEfiVT100PlusGuid;
 extern EFI_GUID gEfiVTUTF8Guid;
 
-extern EFI_GUID NullGuid;
 extern EFI_GUID UnknownDevice;
 
 extern EFI_GUID EfiPartTypeSystemPartitionGuid;
@@ -152,8 +172,10 @@ extern EFI_GUID EfiPartTypeLegacyMbrGuid;
 
 extern EFI_GUID MpsTableGuid;
 extern EFI_GUID AcpiTableGuid;
-extern EFI_GUID SMBIOSTableGuid;
-extern EFI_GUID SMBIOS3TableGuid;
+extern EFI_GUID gEfiSmbiosTableGuid;
+#define SMBIOSTableGuid gEfiSmbiosTableGuid
+extern EFI_GUID gEfiSmbios3TableGuid;
+#define SMBIOS3TableGuid gEfiSmbios3TableGuid
 extern EFI_GUID SalSystemTableGuid;
 extern EFI_GUID EfiDtbTableGuid;
 
@@ -174,21 +196,21 @@ extern EFI_GUID ShellDynamicCommandProtocolGuid;
 //
 #define LOAD_OPTION_ACTIVE      0x00000001
 
-#define VarLanguageCodes       L"LangCodes"
-#define VarLanguage            L"Lang"
-#define VarTimeout             L"Timeout"
-#define VarConsoleInp          L"ConIn"
-#define VarConsoleOut          L"ConOut"
-#define VarErrorOut            L"ErrOut"
-#define VarBootOption          L"Boot%04x"
-#define VarBootOrder           L"BootOrder"
-#define VarBootNext            L"BootNext"
-#define VarBootCurrent         L"BootCurrent"
-#define VarDriverOption        L"Driver%04x"
-#define VarDriverOrder         L"DriverOrder"
-#define VarConsoleInpDev       L"ConInDev"
-#define VarConsoleOutDev       L"ConOutDev"
-#define VarErrorOutDev         L"ErrOutDev"
+#define VarLanguageCodes       u"LangCodes"
+#define VarLanguage            u"Lang"
+#define VarTimeout             u"Timeout"
+#define VarConsoleInp          u"ConIn"
+#define VarConsoleOut          u"ConOut"
+#define VarErrorOut            u"ErrOut"
+#define VarBootOption          u"Boot%04x"
+#define VarBootOrder           u"BootOrder"
+#define VarBootNext            u"BootNext"
+#define VarBootCurrent         u"BootCurrent"
+#define VarDriverOption        u"Driver%04x"
+#define VarDriverOrder         u"DriverOrder"
+#define VarConsoleInpDev       u"ConInDev"
+#define VarConsoleOutDev       u"ConOutDev"
+#define VarErrorOutDev         u"ErrOutDev"
 
 #define LanguageCodeEnglish    "eng"
 
@@ -203,12 +225,10 @@ extern EFI_DEVICE_PATH EndInstanceDevicePath[];
 extern EFI_MEMORY_TYPE PoolAllocationType;
 
 //
-// STATIC - Name is internal to the module
 // INTERNAL - Name is internal to the component (i.e., directory)
 // BOOTSERVCE - Name of a boot service function
 //
 
-#define STATIC
 #define INTERNAL
 #define BOOTSERVICE
 
@@ -281,15 +301,22 @@ ZeroMem (
     IN UINTN     Size
     );
 
-VOID
+VOID EFIAPI
 SetMem (
     IN VOID     *Buffer,
     IN UINTN    Size,
     IN UINT8    Value
     );
 
-VOID
-CopyMem (
+VOID EFIAPI
+CopyMem_1 (
+    IN VOID     *Dest,
+    IN VOID     *Src,
+    IN UINTN    len
+    );
+
+VOID EFIAPI
+CopyMemC (
     IN VOID     *Dest,
     IN CONST VOID     *Src,
     IN UINTN    len
@@ -392,23 +419,30 @@ StrDuplicate (
     );
 
 UINTN
-strlena (
+AsciiStrLen (
     IN CONST CHAR8    *s1
     );
 
 UINTN
-strcmpa (
+AsciiStrCmp (
     IN CONST CHAR8    *s1,
     IN CONST CHAR8    *s2
     );
 
 UINTN
-strncmpa (
+AsciiStrnCmp (
     IN CONST CHAR8    *s1,
     IN CONST CHAR8    *s2,
-    IN UINTN    len
+    IN UINTN          len
     );
 
+//
+// For compatibility with previous gnu-efi versions
+//
+#define strlena AsciiStrLen
+#define strcmpa AsciiStrCmp
+#define strncmpa AsciiStrnCmp
+
 UINTN
 xtoi (
     CONST CHAR16      *str
@@ -473,12 +507,20 @@ ReleaseLock (
     );
 
 
-INTN
-CompareGuid(
-    IN EFI_GUID     *Guid1,
-    IN EFI_GUID     *Guid2
+BOOLEAN
+EFIAPI
+CompareGuid_1 (
+    IN CONST EFI_GUID     *Guid1,
+    IN CONST EFI_GUID     *Guid2
     );
 
+BOOLEAN
+EFIAPI
+IsZeroGuid (
+    IN CONST EFI_GUID     *Guid1
+);
+
+
 VOID *
 AllocatePool (
     IN UINTN     Size
@@ -490,11 +532,12 @@ AllocateZeroPool (
     );
 
 VOID *
-ReallocatePool (
-    IN VOID                 *OldPool,
-    IN UINTN                OldSize,
-    IN UINTN                NewSize
-    );
+EFIAPI
+ReallocatePool_1 (
+        IN UINTN  OldSize,
+        IN UINTN  NewSize,
+        IN VOID   *OldPool  OPTIONAL
+);
 
 VOID
 FreePool (
@@ -535,6 +578,26 @@ VPrint (
     va_list           args
     );
 
+#if !defined(_MSC_VER)
+UINTN
+EFIAPI
+MS_VPrint (
+    IN CONST CHAR16   *fmt,
+    ms_va_list           args
+    );
+#else
+EFI_INTERNAL
+UINTN
+EFIAPI
+MS_VPrint (
+    IN CONST CHAR16   *fmt,
+    ms_va_list           args
+) {
+   return VPrint(fmt, args);
+}
+#endif
+
+
 UINTN
 UnicodeSPrint (
     OUT CHAR16        *Str,
@@ -542,7 +605,7 @@ UnicodeSPrint (
     IN CONST CHAR16   *fmt,
     ...
     );
-
+    
 UINTN
 UnicodeVSPrint (
     OUT CHAR16        *Str,
@@ -551,12 +614,57 @@ UnicodeVSPrint (
     va_list           args
     );
 
+
+#if !defined(_MSC_VER)
+UINTN
+EFIAPI
+MS_UnicodeVSPrint (
+    OUT CHAR16        *Str,
+    IN UINTN          StrSize,
+    IN CONST CHAR16   *fmt,
+    ms_va_list           args
+    );
+#else
+EFI_INTERNAL
+UINTN
+EFIAPI
+MS_UnicodeVSPrint (
+    OUT CHAR16        *Str,
+    IN UINTN          StrSize,
+    IN CONST CHAR16   *fmt,
+    ms_va_list           args
+) {
+    return UnicodeVSPrint(Str, StrSize, fmt, args);
+}
+#endif
+
+
 CHAR16 *
 VPoolPrint (
     IN CONST CHAR16     *fmt,
     va_list             args
     );
 
+#if !defined(_MSC_VER)
+CHAR16 *
+EFIAPI
+MS_VPoolPrint (
+    IN CONST CHAR16     *fmt,
+    ms_va_list             args
+    );
+#else
+EFI_INTERNAL
+CHAR16 *
+EFIAPI
+MS_VPoolPrint (
+    IN CONST CHAR16     *fmt,
+    ms_va_list             args
+) {
+    return VPoolPrint(fmt, args);
+}
+#endif
+
+
 CHAR16 *
 PoolPrint (
     IN CONST CHAR16     *fmt,
@@ -614,6 +722,15 @@ AsciiVSPrint(
     va_list           args
 );
 
+UINTN
+EFIAPI
+MS_AsciiVSPrint(
+    OUT CHAR8         *Str,
+    IN  UINTN         StrSize,
+    IN  CONST CHAR8   *fmt,
+    ms_va_list           args
+);
+
 //
 // For compatibility with previous gnu-efi versions
 //
@@ -1076,4 +1193,8 @@ extern EFI_DEVICE_IO_INTERFACE  *GlobalIoFncs;
 
 #define Port80(_PostCode)   GlobalIoFncs->Io.Write (GlobalIoFncs, IO_UINT16, (UINT64)0x80, 1, &(_PostCode))
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
